@@ -37,17 +37,11 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <linux/cdrom.h>
+#include <sys/sysmacros.h>
 
 #include "libudev.h"
 #include "libudev-private.h"
-
-_printf_(6,0)
-static void log_fn(struct udev *udev, int priority,
-                   const char *file, int line, const char *fn,
-                   const char *format, va_list args)
-{
-        log_metav(priority, file, line, fn, format, args);
-}
+#include "random-util.h"
 
 /* device info */
 static unsigned int cd_cd_rom;
@@ -132,7 +126,7 @@ static bool is_mounted(const char *device)
         return mounted;
 }
 
-static void info_scsi_cmd_err(struct udev *udev, const char *cmd, int err)
+static void info_scsi_cmd_err(struct udev *udev __attribute__((unused)), const char *cmd, int err)
 {
         if (err == -1) {
                 log_debug("%s failed", cmd);
@@ -150,7 +144,7 @@ struct scsi_cmd {
         struct sg_io_hdr sg_io;
 };
 
-static void scsi_cmd_init(struct udev *udev, struct scsi_cmd *cmd)
+static void scsi_cmd_init(struct udev *udev __attribute__((unused)), struct scsi_cmd *cmd)
 {
         memzero(cmd, sizeof(struct scsi_cmd));
         cmd->cgc.quiet = 1;
@@ -162,7 +156,7 @@ static void scsi_cmd_init(struct udev *udev, struct scsi_cmd *cmd)
         cmd->sg_io.flags = SG_FLAG_LUN_INHIBIT | SG_FLAG_DIRECT_IO;
 }
 
-static void scsi_cmd_set(struct udev *udev, struct scsi_cmd *cmd, size_t i, unsigned char arg)
+static void scsi_cmd_set(struct udev *udev __attribute__((unused)), struct scsi_cmd *cmd, size_t i, unsigned char arg)
 {
         cmd->sg_io.cmd_len = i + 1;
         cmd->cgc.cmd[i] = arg;
@@ -170,7 +164,7 @@ static void scsi_cmd_set(struct udev *udev, struct scsi_cmd *cmd, size_t i, unsi
 
 #define CHECK_CONDITION 0x01
 
-static int scsi_cmd_run(struct udev *udev, struct scsi_cmd *cmd, int fd, unsigned char *buf, size_t bufsize)
+static int scsi_cmd_run(struct udev *udev __attribute__((unused)), struct scsi_cmd *cmd, int fd, unsigned char *buf, size_t bufsize)
 {
         int ret = 0;
 
@@ -196,7 +190,7 @@ static int scsi_cmd_run(struct udev *udev, struct scsi_cmd *cmd, int fd, unsigne
         return ret;
 }
 
-static int media_lock(struct udev *udev, int fd, bool lock)
+static int media_lock(struct udev *udev __attribute__((unused)), int fd, bool lock)
 {
         int err;
 
@@ -229,7 +223,7 @@ static int media_eject(struct udev *udev, int fd)
         return 0;
 }
 
-static int cd_capability_compat(struct udev *udev, int fd)
+static int cd_capability_compat(struct udev *udev __attribute__((unused)), int fd)
 {
         int capability;
 
@@ -256,7 +250,7 @@ static int cd_capability_compat(struct udev *udev, int fd)
         return 0;
 }
 
-static int cd_media_compat(struct udev *udev, int fd)
+static int cd_media_compat(struct udev *udev __attribute__((unused)), int fd)
 {
         if (ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT) != CDS_DISC_OK) {
                 log_debug("CDROM_DRIVE_STATUS != CDS_DISC_OK");
@@ -291,7 +285,7 @@ static int cd_inquiry(struct udev *udev, int fd)
         return 0;
 }
 
-static void feature_profile_media(struct udev *udev, int cur_profile)
+static void feature_profile_media(struct udev *udev __attribute__((unused)), int cur_profile)
 {
         switch (cur_profile) {
         case 0x03:
@@ -400,7 +394,7 @@ static void feature_profile_media(struct udev *udev, int cur_profile)
         }
 }
 
-static int feature_profiles(struct udev *udev, const unsigned char *profiles, size_t size)
+static int feature_profiles(struct udev *udev __attribute__((unused)), const unsigned char *profiles, size_t size)
 {
         unsigned int i;
 
@@ -651,7 +645,7 @@ static int cd_media_info(struct udev *udev, int fd)
         if (!cd_media_cd_rom)
                 cd_media_state = media_status[header[2] & 3];
 
-        /* fresh DVD-RW in restricted overwite mode reports itself as
+        /* fresh DVD-RW in restricted overwrite mode reports itself as
          * "appendable"; change it to "blank" to make it consistent with what
          * gets reported after blanking, and what userspace expects  */
         if (cd_media_dvd_rw_ro && (header[2] & 3) == 1)
@@ -662,8 +656,8 @@ static int cd_media_info(struct udev *udev, int fd)
          * write protected; we need to check the contents if it is blank */
         if ((cd_media_dvd_rw_ro || cd_media_dvd_plus_rw || cd_media_dvd_plus_rw_dl || cd_media_dvd_ram) && (header[2] & 3) > 1) {
                 unsigned char buffer[32 * 2048];
-                unsigned char result, len;
-                int block, offset;
+                unsigned char len;
+                int offset;
 
                 if (cd_media_dvd_ram) {
                         /* a write protected dvd-ram may report "complete" status */
@@ -744,22 +738,23 @@ static int cd_media_info(struct udev *udev, int fd)
                 /* if any non-zero data is found in sector 16 (iso and udf) or
                  * eventually 0 (fat32 boot sector, ext2 superblock, etc), disc
                  * is assumed non-blank */
-                result = 0;
 
-                for (block = 32768; block >= 0 && !result; block -= 32768) {
-                        offset = block;
-                        while (offset < (block + 2048) && !result) {
-                                result = buffer [offset];
-                                offset++;
+                for (offset = 32768; offset < (32768 + 2048); offset++) {
+                        if (buffer [offset]) {
+                                log_debug("data in block 16, assuming complete");
+                                goto determined;
                         }
                 }
 
-                if (!result) {
-                        cd_media_state = media_status[0];
-                        log_debug("no data in blocks 0 or 16, assuming blank");
-                } else {
-                        log_debug("data in blocks 0 or 16, assuming complete");
+                for (offset = 0; offset < 2048; offset++) {
+                        if (buffer [offset]) {
+                                log_debug("data in block 0, assuming complete");
+                                goto determined;
+                        }
                 }
+
+                cd_media_state = media_status[0];
+                log_debug("no data in blocks 0 or 16, assuming blank");
         }
 
 determined:
@@ -878,8 +873,6 @@ int main(int argc, char *argv[])
         if (udev == NULL)
                 goto exit;
 
-        udev_set_log_fn(udev, log_fn);
-
         while (1) {
                 int option;
 
@@ -900,16 +893,15 @@ int main(int argc, char *argv[])
                 case 'd':
                         log_set_target(LOG_TARGET_CONSOLE);
                         log_set_max_level(LOG_DEBUG);
-                        udev_set_log_priority(udev, LOG_DEBUG);
                         log_open();
                         break;
                 case 'h':
                         printf("Usage: cdrom_id [options] <device>\n"
-                               "  --lock-media    lock the media (to enable eject request events)\n"
-                               "  --unlock-media  unlock the media\n"
-                               "  --eject-media   eject the media\n"
-                               "  --debug         debug to stderr\n"
-                               "  --help          print this help text\n\n");
+                               "  -l,--lock-media    lock the media (to enable eject request events)\n"
+                               "  -u,--unlock-media  unlock the media\n"
+                               "  -e,--eject-media   eject the media\n"
+                               "  -d,--debug         debug to stderr\n"
+                               "  -h,--help          print this help text\n\n");
                         goto exit;
                 default:
                         rc = 1;
@@ -925,7 +917,7 @@ int main(int argc, char *argv[])
                 goto exit;
         }
 
-        srand((unsigned int)getpid());
+        initialize_srand();
         for (cnt = 20; cnt > 0; cnt--) {
                 struct timespec duration;
 
@@ -1077,17 +1069,17 @@ work:
         if (cd_media_state != NULL)
                 printf("ID_CDROM_MEDIA_STATE=%s\n", cd_media_state);
         if (cd_media_session_next > 0)
-                printf("ID_CDROM_MEDIA_SESSION_NEXT=%d\n", cd_media_session_next);
+                printf("ID_CDROM_MEDIA_SESSION_NEXT=%u\n", cd_media_session_next);
         if (cd_media_session_count > 0)
-                printf("ID_CDROM_MEDIA_SESSION_COUNT=%d\n", cd_media_session_count);
+                printf("ID_CDROM_MEDIA_SESSION_COUNT=%u\n", cd_media_session_count);
         if (cd_media_session_count > 1 && cd_media_session_last_offset > 0)
                 printf("ID_CDROM_MEDIA_SESSION_LAST_OFFSET=%llu\n", cd_media_session_last_offset);
         if (cd_media_track_count > 0)
-                printf("ID_CDROM_MEDIA_TRACK_COUNT=%d\n", cd_media_track_count);
+                printf("ID_CDROM_MEDIA_TRACK_COUNT=%u\n", cd_media_track_count);
         if (cd_media_track_count_audio > 0)
-                printf("ID_CDROM_MEDIA_TRACK_COUNT_AUDIO=%d\n", cd_media_track_count_audio);
+                printf("ID_CDROM_MEDIA_TRACK_COUNT_AUDIO=%u\n", cd_media_track_count_audio);
         if (cd_media_track_count_data > 0)
-                printf("ID_CDROM_MEDIA_TRACK_COUNT_DATA=%d\n", cd_media_track_count_data);
+                printf("ID_CDROM_MEDIA_TRACK_COUNT_DATA=%u\n", cd_media_track_count_data);
 exit:
         if (fd >= 0)
                 close(fd);
