@@ -26,6 +26,7 @@
 #include <string.h>
 #include <limits.h>
 #include <sys/stat.h>
+#include <sys/inotify.h>
 
 #include "libudev.h"
 #include "libudev-private.h"
@@ -45,6 +46,7 @@
 struct udev_queue {
         struct udev *udev;
         int refcount;
+        int fd;
 };
 
 /**
@@ -69,6 +71,7 @@ _public_ struct udev_queue *udev_queue_new(struct udev *udev)
 
         udev_queue->refcount = 1;
         udev_queue->udev = udev;
+        udev_queue->fd = -1;
         return udev_queue;
 }
 
@@ -107,6 +110,8 @@ _public_ struct udev_queue *udev_queue_unref(struct udev_queue *udev_queue)
         if (udev_queue->refcount > 0)
                 return NULL;
 
+        safe_close(udev_queue->fd);
+
         free(udev_queue);
         return NULL;
 }
@@ -134,7 +139,7 @@ _public_ struct udev *udev_queue_get_udev(struct udev_queue *udev_queue)
  *
  * Returns: 0.
  **/
-_public_ unsigned long long int udev_queue_get_kernel_seqnum(struct udev_queue *udev_queue)
+_public_ unsigned long long int udev_queue_get_kernel_seqnum(struct udev_queue *udev_queue __attribute__((unused)))
 {
         return 0;
 }
@@ -147,7 +152,7 @@ _public_ unsigned long long int udev_queue_get_kernel_seqnum(struct udev_queue *
  *
  * Returns: 0.
  **/
-_public_ unsigned long long int udev_queue_get_udev_seqnum(struct udev_queue *udev_queue)
+_public_ unsigned long long int udev_queue_get_udev_seqnum(struct udev_queue *udev_queue __attribute__((unused)))
 {
         return 0;
 }
@@ -160,9 +165,9 @@ _public_ unsigned long long int udev_queue_get_udev_seqnum(struct udev_queue *ud
  *
  * Returns: a flag indicating if udev is active.
  **/
-_public_ int udev_queue_get_udev_is_active(struct udev_queue *udev_queue)
+_public_ int udev_queue_get_udev_is_active(struct udev_queue *udev_queue __attribute__((unused)))
 {
-        return access("/run/udev/control", F_OK) >= 0;
+        return access(UDEV_ROOT_RUN "/udev/control", F_OK) >= 0;
 }
 
 /**
@@ -173,9 +178,9 @@ _public_ int udev_queue_get_udev_is_active(struct udev_queue *udev_queue)
  *
  * Returns: a flag indicating if udev is currently handling events.
  **/
-_public_ int udev_queue_get_queue_is_empty(struct udev_queue *udev_queue)
+_public_ int udev_queue_get_queue_is_empty(struct udev_queue *udev_queue __attribute__((unused)))
 {
-        return access("/run/udev/queue", F_OK) >= 0;
+        return access(UDEV_ROOT_RUN "/udev/queue", F_OK) < 0;
 }
 
 /**
@@ -190,7 +195,7 @@ _public_ int udev_queue_get_queue_is_empty(struct udev_queue *udev_queue)
  * Returns: a flag indicating if udev is currently handling events.
  **/
 _public_ int udev_queue_get_seqnum_sequence_is_finished(struct udev_queue *udev_queue,
-                                               unsigned long long int start, unsigned long long int end)
+                                               unsigned long long int start __attribute__((unused)), unsigned long long int end __attribute__((unused)))
 {
         return udev_queue_get_queue_is_empty(udev_queue);
 }
@@ -205,7 +210,7 @@ _public_ int udev_queue_get_seqnum_sequence_is_finished(struct udev_queue *udev_
  *
  * Returns: a flag indicating if udev is currently handling events.
  **/
-_public_ int udev_queue_get_seqnum_is_finished(struct udev_queue *udev_queue, unsigned long long int seqnum)
+_public_ int udev_queue_get_seqnum_is_finished(struct udev_queue *udev_queue, unsigned long long int seqnum __attribute__((unused)))
 {
         return udev_queue_get_queue_is_empty(udev_queue);
 }
@@ -218,7 +223,48 @@ _public_ int udev_queue_get_seqnum_is_finished(struct udev_queue *udev_queue, un
  *
  * Returns: NULL.
  **/
-_public_ struct udev_list_entry *udev_queue_get_queued_list_entry(struct udev_queue *udev_queue)
+_public_ struct udev_list_entry *udev_queue_get_queued_list_entry(struct udev_queue *udev_queue __attribute__((unused)))
 {
         return NULL;
 }
+
+/**
+ * udev_queue_get_fd:
+ * @udev_queue: udev queue context
+ *
+ * Returns: a file descriptor to watch for a queue to become empty.
+ */
+_public_ int udev_queue_get_fd(struct udev_queue *udev_queue) {
+        int fd;
+        int r;
+
+        if (udev_queue->fd >= 0)
+                return udev_queue->fd;
+
+        fd = inotify_init1(IN_CLOEXEC);
+        if (fd < 0)
+                return -errno;
+
+        r = inotify_add_watch(fd, UDEV_ROOT_RUN "/udev" , IN_DELETE);
+        if (r < 0) {
+                r = -errno;
+                close(fd);
+                return r;
+        }
+
+        udev_queue->fd = fd;
+        return fd;
+}
+
+/**
+ * udev_queue_flush:
+ * @udev_queue: udev queue context
+ *
+ * Returns: the result of clearing the watch for queue changes.
+ */
+_public_ int udev_queue_flush(struct udev_queue *udev_queue) {
+        if (udev_queue->fd < 0)
+                return -EINVAL;
+
+        return flush_fd(udev_queue->fd);
+}
