@@ -27,12 +27,14 @@
 #include "utf8.h"
 
 int whitelisted_char_for_devnode(char c, const char *white) {
+
         if ((c >= '0' && c <= '9') ||
             (c >= 'A' && c <= 'Z') ||
             (c >= 'a' && c <= 'z') ||
             strchr("#+-.:=@_", c) != NULL ||
             (white != NULL && strchr(white, c) != NULL))
                 return 1;
+
         return 0;
 }
 
@@ -47,27 +49,34 @@ int encode_devnode_name(const char *str, char *str_enc, size_t len) {
 
                 seqlen = utf8_encoded_valid_unichar(&str[i]);
                 if (seqlen > 1) {
+
                         if (len-j < (size_t)seqlen)
-                                goto err;
+                                return -EINVAL;
+
                         memcpy(&str_enc[j], &str[i], seqlen);
                         j += seqlen;
                         i += (seqlen-1);
+
                 } else if (str[i] == '\\' || !whitelisted_char_for_devnode(str[i], NULL)) {
+
                         if (len-j < 4)
-                                goto err;
+                                return -EINVAL;
+
                         sprintf(&str_enc[j], "\\x%02x", (unsigned char) str[i]);
                         j += 4;
+
                 } else {
                         if (len-j < 1)
-                                goto err;
+                                return -EINVAL;
+
                         str_enc[j] = str[i];
                         j++;
                 }
         }
+
         if (len-j < 1)
-                goto err;
+                return -EINVAL;
+
         str_enc[j] = '\0';
         return 0;
-err:
-        return -EINVAL;
 }
