@@ -17,6 +17,8 @@
   along with systemd; If not, see <http://www.gnu.org/licenses/>.
 ***/
 
+#pragma once
+
 #include <stdbool.h>
 
 #include "macro.h"
@@ -28,11 +30,19 @@ char* path_make_absolute(const char *p, const char *prefix);
 char* path_make_absolute_cwd(const char *p);
 char* path_kill_slashes(char *path);
 char* path_startswith(const char *path, const char *prefix) _pure_;
+int path_compare(const char *a, const char *b) _pure_;
 bool path_equal(const char *a, const char *b) _pure_;
 
 char** path_strv_resolve(char **l, const char *prefix);
 char** path_strv_resolve_uniq(char **l, const char *prefix);
 
+int fd_is_mount_point(int fd);
 int path_is_mount_point(const char *path, bool allow_symlink);
 bool paths_check_timestamp(const char* const* paths, usec_t *paths_ts_usec, bool update);
 
+char *prefix_root(const char *root, const char *path);
+
+char* path_extend_internal(char **x, ...);
+#define path_extend(x, ...) path_extend_internal(x, __VA_ARGS__, POINTER_MAX)
+#define path_join(...) path_extend_internal(NULL, __VA_ARGS__, POINTER_MAX)
+
