@@ -24,7 +24,9 @@
 #endif
 
 #include "util.h"
+#include "process-util.h"
 #include "path-util.h"
+#include "fileio.h"
 #include "smack-util.h"
 
 #define SMACK_FLOOR_LABEL "_"
@@ -43,7 +45,7 @@ bool mac_smack_use(void) {
 #endif
 }
 
-int mac_smack_apply(const char *path, const char *label) {
+int mac_smack_apply(const char *path, const char *label __attribute__((unused))) {
         int r = 0;
 
         assert(path);
@@ -63,7 +65,7 @@ int mac_smack_apply(const char *path, const char *label) {
         return r;
 }
 
-int mac_smack_apply_fd(int fd, const char *label) {
+int mac_smack_apply_fd(int fd, const char *label __attribute__((unused))) {
         int r = 0;
 
         assert(fd >= 0);
@@ -83,7 +85,7 @@ int mac_smack_apply_fd(int fd, const char *label) {
         return r;
 }
 
-int mac_smack_apply_ip_out_fd(int fd, const char *label) {
+int mac_smack_apply_ip_out_fd(int fd, const char *label __attribute__((unused))) {
         int r = 0;
 
         assert(fd >= 0);
@@ -103,7 +105,7 @@ int mac_smack_apply_ip_out_fd(int fd, const char *label) {
         return r;
 }
 
-int mac_smack_apply_ip_in_fd(int fd, const char *label) {
+int mac_smack_apply_ip_in_fd(int fd, const char *label __attribute__((unused))) {
         int r = 0;
 
         assert(fd >= 0);
@@ -123,14 +125,38 @@ int mac_smack_apply_ip_in_fd(int fd, const char *label) {
         return r;
 }
 
-int mac_smack_fix(const char *path, bool ignore_enoent, bool ignore_erofs) {
+int mac_smack_apply_pid(pid_t pid __attribute__((unused)), const char *label) {
+
+#ifdef HAVE_SMACK
+        const char *p;
+#endif
         int r = 0;
 
+        assert(label);
+
+#ifdef HAVE_SMACK
+        if (!mac_smack_use())
+                return 0;
+
+        p = procfs_file_alloca(pid, "attr/current");
+        r = write_string_file(p, label);
+        if (r < 0)
+                return r;
+#endif
+
+        return r;
+}
+
+int mac_smack_fix(const char *path, bool ignore_enoent __attribute__((unused)), bool ignore_erofs __attribute__((unused))) {
+
 #ifdef HAVE_SMACK
         struct stat st;
+#endif
+        int r = 0;
 
         assert(path);
 
+#ifdef HAVE_SMACK
         if (!mac_smack_use())
                 return 0;
 
@@ -162,7 +188,7 @@ int mac_smack_fix(const char *path, bool ignore_enoent, bool ignore_erofs) {
                 r = lsetxattr(path, "security.SMACK64", label, strlen(label), 0);
 
                 /* If the FS doesn't support labels, then exit without warning */
-                if (r < 0 && errno == ENOTSUP)
+                if (r < 0 && errno == EOPNOTSUPP)
                         return 0;
         }
 
@@ -174,8 +200,7 @@ int mac_smack_fix(const char *path, bool ignore_enoent, bool ignore_erofs) {
                 if (ignore_erofs && errno == EROFS)
                         return 0;
 
-                log_debug("Unable to fix SMACK label of %s: %m", path);
-                r = -errno;
+                r = log_debug_errno(errno, "Unable to fix SMACK label of %s: %m", path);
         }
 #endif
 
