@@ -50,7 +50,6 @@
 #include <sys/vfs.h>
 #include <linux/magic.h>
 #include <limits.h>
-#include <langinfo.h>
 #include <locale.h>
 #include <libgen.h>
 
@@ -1726,7 +1725,7 @@ int tempfn_xxxxxx(const char *p, char **ret) {
          *         /foo/bar/.#waldoXXXXXX
          */
 
-        fn = basename((char*)p);
+        fn = eudev_basename(p);
         if (!filename_is_valid(fn))
                 return -EINVAL;
 
@@ -1968,3 +1967,11 @@ void cmsg_close_all(struct msghdr *mh) {
                 if (cmsg->cmsg_level == SOL_SOCKET && cmsg->cmsg_type == SCM_RIGHTS)
                         close_many((int*) CMSG_DATA(cmsg), (cmsg->cmsg_len - CMSG_LEN(0)) / sizeof(int));
 }
+
+const char *eudev_basename(const char *filename) {
+        const char *p = strrchr(filename, '/');
+
+        if (p)
+                return p + 1;
+        return filename;
+}
