@@ -46,10 +46,6 @@ static inline int unshare (int x) { return syscall(SYS_unshare, x); }
 #include <linux/sched.h>
 #endif
 
-void udev_main_log(struct udev *udev, int priority,
-                   const char *file, int line, const char *fn,
-                   const char *format, va_list args) {}
-
 static int fake_filesystems(void) {
         static const struct fakefs {
                 const char *src;
@@ -58,7 +54,7 @@ static int fake_filesystems(void) {
         } fakefss[] = {
                 { "test/sys", "/sys",                   "failed to mount test /sys" },
                 { "test/dev", "/dev",                   "failed to mount test /dev" },
-                { "test/run", "/run",                   "failed to mount test /run" },
+                { "test/run", UDEV_ROOT_RUN,            "failed to mount test " UDEV_ROOT_RUN },
                 { "test/run", "/etc/udev/rules.d",      "failed to mount empty /etc/udev/rules.d" },
                 { "test/run", "/lib/udev/rules.d",      "failed to mount empty /lib/udev/rules.d" },
         };
@@ -90,7 +86,7 @@ out:
         return err;
 }
 
-int main(int argc, char *argv[]) {
+int main(int argc __attribute__((unused)), char *argv[]) {
         _cleanup_udev_unref_ struct udev *udev = NULL;
         _cleanup_udev_event_unref_ struct udev_event *event = NULL;
         _cleanup_udev_device_unref_ struct udev_device *dev = NULL;
@@ -129,13 +125,12 @@ int main(int argc, char *argv[]) {
         rules = udev_rules_new(udev, 1);
 
         strscpyl(syspath, sizeof(syspath), "/sys", devpath, NULL);
-        dev = udev_device_new_from_syspath(udev, syspath);
+        dev = udev_device_new_from_synthetic_event(udev, syspath, action);
         if (dev == NULL) {
                 log_debug("unknown device '%s'", devpath);
                 goto out;
         }
 
-        udev_device_set_action(dev, action);
         event = udev_event_new(dev);
 
         sigfillset(&mask);
@@ -164,8 +159,14 @@ int main(int argc, char *argv[]) {
                 }
         }
 
-        udev_event_execute_rules(event, 3 * USEC_PER_SEC, USEC_PER_SEC, rules, &sigmask_orig);
-        udev_event_execute_run(event, 3 * USEC_PER_SEC, USEC_PER_SEC, NULL);
+        udev_event_execute_rules(event,
+                                 3 * USEC_PER_SEC, USEC_PER_SEC,
+                                 NULL,
+                                 rules,
+                                 &sigmask_orig);
+        udev_event_execute_run(event,
+                               3 * USEC_PER_SEC, USEC_PER_SEC,
+                               NULL);
 out:
         if (event != NULL && event->fd_signal >= 0)
                 close(event->fd_signal);
